from surveyhelper.question import *
from surveyhelper.codebook import *
from surveyhelper.qsf_parser import *
from surveyhelper.response_set import *
from surveyhelper.frequency_report import *
from surveyhelper.scale import *

import os
import glob

modules = glob.glob(os.path.dirname(__file__)+"/*.py")
__all__ = [ os.path.basename(f)[:-3] for f in modules]


import pandas as pd
import numpy as np

class ResponseSet:

    # TODO - As currently written this code skips the second row of data,
    # because that's what we want to do for Qualtrics csv results, but this is
    # potentially a big gotcha, so document well or change.
    def __init__(self, response_file, codebook, 
                 skiprows = [1], 
                 encoding="utf8",
                 grouping_var=None,
                 group_dict=None):
        df = pd.read_csv(response_file , skiprows=skiprows, encoding=encoding)
        # go through each variable in the codebook and make sure the corresponding 
        # column is integer coded
        matched_questions = []
        for q in codebook.get_questions():
            matched = True
            for v in q.get_variable_names():
                if v not in df:
                    print("Warning: Expected variable {} not found in data file {}".format(v, response_file))
                    matched = False
                elif df[v].dtype not in [np.int64, np.float64]:
                    print("Converting variable {} to integer from {}".format(v, df[v].dtype))
                    df[v] = df[v].convert_objects(convert_numeric=True)
            if matched: matched_questions.append(q)
        self.data = df
        self.matched_questions = matched_questions
        self.codebook = codebook
        self.grouping_var = grouping_var
        if (not self.grouping_var and group_dict):
            raise(Exception(
                  "Grouping variable must also be specified when a grouping dict is passed in."
                 ))
        if self.grouping_var and not group_dict:
            self.data[grouping_var] = pd.Categorical(self.data[grouping_var])
        if self.grouping_var and group_dict:
                self.data[grouping_var] = self.data[grouping_var].astype(str)
                # For some odd reason, inplace=True doesn't work when cats are ["1-2", "3+"]
                self.data[grouping_var] = self.data[grouping_var].replace(group_dict)
                self.data[grouping_var] = pd.Categorical(self.data[grouping_var])
                self.data[grouping_var] = self.data[grouping_var].cat.reorder_categories(self.uniq(list(group_dict.values())))

    def uniq(self, input):
        output = []
        for x in input:
            if x not in output:
                output.append(x)
        return(output)


    def get_data(self):
        if not self.grouping_var:
            group_var = 'z'
            while group_var in self.data.columns:
                group_var += 'z'
            self.data[group_var] = 0
        else:
            group_var = self.grouping_var
            # self.data.sort(group_var, inplace=True)
        groups = self.data.groupby(group_var)
        return(groups)

    


from itertools import compress
import pandas as pd
import numpy as np
import json
from abc import ABCMeta, abstractmethod
from surveyhelper.scale import QuestionScale, LikertScale, NominalScale, OrdinalScale
from scipy.stats import ttest_ind, f_oneway, chisquare

class MatrixQuestion:
    __metaclass__ = ABCMeta

    def __init__(self, text, label, questions):
        self.text = text
        self.questions = questions
        self.label = label
        self.assert_questions_same_type()
        self.assert_choices_same()
        self.assign_children_to_matrix()

    def exclude_choices_from_analysis(self, choices):
        for q in self.questions:
            q.exclude_choices_from_analysis(choices)

    def reverse_choices(self):
        for q in self.questions:
            q.reverse_choices()

    def change_scale(self, newtype, 
                     values = None, 
                     midpoint = None):
        for q in self.questions:
            q.change_scale(newtype, values, midpoint)

    def change_midpoint(self, midpoint):
        for q in self.questions:
            q.scale.midpoint = midpoint

    def get_scale(self):
        if len(self.questions) > 0:
            return(self.questions[0].scale)
        else:
            None

    def assert_questions_same_type(self):
        if all(type(x) == type(self.questions[0]) for x in self.questions):
            return(True)
        else:
            raise(Exception("Questions in a matrix must all have the same type"))

    def assert_choices_same(self):
        if all([x.scale == self.questions[0].scale for x in self.questions]):
            return(True)
        else:
            raise(Exception("Questions in a matrix must all have the same choices"))

    def assign_children_to_matrix(self):
        for q in self.questions:
            q.matrix = self
        return

    def get_variable_names(self):
        names = []
        for q in self.questions:
            names += q.get_variable_names()
        return(names)

    def get_children_text(self):
        return([q.text for q in self.questions])

    def pretty_print(self, 
                     show_choices=True):
        print("{} ({})".format(self.text, self.label))
        if show_choices:
            self.questions[0].pretty_print_choices()
        for q in self.questions:
            print(q.text)

    @abstractmethod
    def get_choices(self):
        pass

    @abstractmethod
    def frequency_table(self):
        pass

    def freq_table_to_json(self, df):
        return('')

class SelectOneMatrixQuestion(MatrixQuestion):

    def get_choices(self, 
                    remove_exclusions=True, 
                    show_values=False):
        self.assert_choices_same()
        if len(self.questions) > 0:
            return(self.questions[0].scale.choices_to_str(remove_exclusions,
                                                          show_values))
        else:
            return([])

    def frequency_table(self, df, 
                        show="ct", 
                        pct_format=".0%",
                        remove_exclusions = True, 
                        show_totals=True, 
                        remove_zero_totals=True, 
                        show_mean=True, 
                        mean_format=".1f"):
        if len(self.questions) == 0:
            return(pd.DataFrame())
        data = []
        # TODO: better way to do this without repeatedly calling frequency_table
        # whole method could be way more efficient
        questions_to_drop = {}
        if show == "ct":
            for q in self.questions:
                f = q.frequency_table(df, False, True, False, pct_format, 
                    remove_exclusions, show_totals, remove_zero_totals, 
                    show_mean, mean_format
                    ).iloc[:,0].tolist()
                if len(f) > 0:
                    data.append(f)
                else:
                    questions_to_drop[q.text] = True
        elif show == "pct":
            for q in self.questions:
                f = q.frequency_table(df, False, False, True, pct_format, 
                    remove_exclusions, show_totals, remove_zero_totals, 
                    show_mean, mean_format
                    ).iloc[:,0].tolist()
                if len(f) > 0:
                    data.append(f)
                else:
                    questions_to_drop[q.text] = True
        else:
            raise(Exception("Invalid 'show' parameter: {}".format(show)))
        tbl = pd.DataFrame(data)
        tmpcols = self.get_choices(remove_exclusions)

        if show_totals:
            tmpcols.append("Total")
        if show_mean:
            tmpcols.append("Mean")
        tbl.columns = tmpcols
        qs = self.get_children_text()
        new_qs = []
        [new_qs.append(q) for q in qs if not q in questions_to_drop]
        tbl["Question"] = qs
        cols = tbl.columns.tolist()
        cols = cols[-1:] + cols[:-1]
        tbl = tbl[cols]
        return(tbl)

    def cut_by_question(self, other_question, response_set, 
                        cut_var_label=None,
                        question_labels=None,
                        pct_format=".0%",
                        remove_exclusions=True, 
                        remove_zero_totals=True,
                        show_mean=True, 
                        mean_format=".1f"):
        if type(other_question) != SelectOneQuestion:
            raise(Exception("Can only call cut_by_question on a SelectOneQuestion type"))
        groups = response_set.groupby(other_question.label)
        group_mapping = dict(zip(other_question.values, other_question.choices))
        oth_text = cut_var_label
        if not oth_text:
            oth_text = other_question.text
        return(self.cut_by(groups, group_mapping, oth_text, 
                           question_labels=question_labels,
                           pct_format=pct_format,
                           remove_exclusions=remove_exclusions,
                           remove_zero_totals=remove_zero_totals,
                           show_mean=show_mean, 
                           mean_format=mean_format))

    def cut_by(self, groups, group_label_mapping, cut_var_label, 
               question_labels=None, 
               pct_format=".0%",
               remove_exclusions=True, 
               remove_zero_totals=True, 
               show_mean=True,
               mean_format=".1f"):
        results = []
        labels = question_labels
        if not labels:
            labels = [q.text for q in self.questions]
        for q, l in zip(self.questions, labels):
            r = q.cut_by(groups, cut_var_label, 
                         group_label_mapping = group_label_mapping,
                         freq_table_options={
                            "ct":True, 
                            "pct":False,
                            "pct_format": pct_format, 
                            "remove_exclusions": remove_exclusions, 
                            "show_totals":False,
                            "remove_zero_totals": remove_zero_totals,
                            "show_mean": show_mean,
                            "mean_format": mean_format,
                            "show_values":False                         
                         })
            results.append(r.T)
        return(pd.concat(results))

    def freq_table_to_json(self, df):
        t = self.frequency_table(df, "ct", "", True, False, False, "")
        # If all the entries in the table are zero, return empty string
        if t.ix[:,1:].sum().sum() == 0:
            return('')
        else:
            t.set_index('Question', inplace=True)
            t.columns = t.columns.astype(str)
            return(t.to_json(orient="split"))

    def cut_by_json(self, rs, mark_sig_diffs=False, sort_by_mean=False):
        tables = []
        data = rs.get_data()
        group_var = rs.grouping_var
        for q in self.questions:
            t = q.cut_by(data, group_var, 
                freq_table_options={"show_question":True, 
                                    "ct":True, 
                                    "pct":False,
                                    "pct_format":"", 
                                    "remove_exclusions":True, 
                                    "show_totals":False,
                                    "remove_zero_totals":True,
                                    "show_mean":True,
                                    "mean_format":"10",
                                    "show_values":False                         
                                   })
            meancol = "Mean"
            suffix = ""
            if "Mean*" in t.columns:
                suffix = '*'
                meancol = meancol + suffix
            if mark_sig_diffs:
                t.index = [q.text + ":" + str(i) + suffix for i in t.index]
            t.drop(meancol, axis=1, inplace=True)
            if t.ix[:,1:-1].sum().sum() != 0:
                tables.append(t)
        if len(tables)==0:
            return('')
        else:
            df = pd.concat(tables)
            df.columns = df.columns.astype(str)
            return(df.to_json(orient="split"))

    def graph_type(self, num_groups=1):
        if len(self.questions) > 0:
            if type(self.questions[0].scale) == LikertScale:
                if num_groups > 1:
                    return('grouped_diverging_bar')
                else:
                    return('diverging_bar')
            else:
                if num_groups > 1:
                    return('grouped_bar')
                else:
                    return('horizontal_stacked_bar')
        else:
            return('')

class SelectMultipleMatrixQuestion(MatrixQuestion):

    def get_choices(self, remove_exclusions=True):
        self.assert_choices_same()
        if len(self.questions > 0):
            return(self.questions[0].get_choices(remove_exclusions))
        else:
            []

    def frequency_table(self, df, 
                        show="ct", 
                        pct_format=".0%",
                        remove_exclusions = True, 
                        show_totals=True):
        data = []
        if show == "ct":
            for q in self.questions:
                data.append(q.frequency_table(df, False, True,
                            False, False, pct_format, remove_exclusions,
                            False).iloc[:,0].tolist())
        elif show == "pct_respondents":
            for q in self.responses:
                data.append(q.frequency_table(df, False, False, 
                            True, False, pct_format, remove_exclusions,
                            False).iloc[:,0].tolist())
        elif show == "pct_responses":
            for q in self.responses:
                data.append(q.frequency_table(df, False, False, 
                            False, True, pct_format, remove_exclusions,
                            False).iloc[:,0].tolist())
        else:
            raise(Exception("Invalid 'show' parameter: {}".format(show)))       
        tbl = pd.DataFrame(data)

        tbl.columns = self.get_choices(remove_exclusions)
        tbl["Question"] = self.get_children_text()
        cols = tbl.columns.tolist()
        cols = cols[-1:] + cols[:-1]
        tbl = tbl[cols]
        if show_totals:
            tots = []
            for q in self.questions:
                tots.append(q.get_total_respondents(df))
            tbl["Total Respondents"] = tots
        return(tbl)

class SelectQuestion:
    __metaclass__ = ABCMeta

    def get_total_respondents(self, df):
        freqs, resp, nonresp = self.tally(df)
        return(resp)

    def get_scale(self):
        return(self.scale)

    def change_scale(self, newtype, values = None, midpoint = None):
        self.scale = QuestionScale.change_scale(self.scale, newtype)

    def change_midpoint(self, midpoint):
        self.scale.midpoint = midpoint

    def exclude_choices_from_analysis(self, choices):
        self.scale.exclude_choices_from_analysis(choices)

    @abstractmethod
    def get_variable_names(self):
        pass

    @abstractmethod
    def pretty_print(self):
        pass

    @abstractmethod
    def pretty_print_choices(self):
        pass

    @abstractmethod
    def tally(self):
        pass

    @abstractmethod
    def frequency_table(self):
        pass

    def questions_to_json(self):
        return('')

class SelectOneQuestion(SelectQuestion):

    def __init__(self, text, var, choices, label, values, 
                 exclude_from_analysis, 
                 matrix=None, 
                 scale_type='likert'):
        self.text = text
        self.label = label
        self.variable = var
        self.matrix = matrix
        self.scale = QuestionScale.create_scale(scale_type, choices, 
                        exclude_from_analysis, values)

    def get_variable_names(self):
        return([self.variable])

    def pretty_print(self, 
                     show_choices=True):
        print("{} ({})".format(self.text, self.label))
        if show_choices:
            self.pretty_print_choices()

    def pretty_print_choices(self):
        print(", ".join(self.scale.choices_to_str(False)))

    def reverse_choices(self):
        self.scale.reverse_choices()

    def mean(self, df, 
             remove_exclusions=True):
        values = self.scale.get_values(remove_exclusions)
        freq, n, x = self.tally(df, remove_exclusions)
        num = sum([ct * v for ct, v in zip(freq, values)])
        if n > 0:
            return(num/n)
        else:
            return(np.nan)

    def tally(self, df, 
              remove_exclusions=True):
        """
        Returns ([response frequencies], respondents, nonrespondents) 
        tuple where response frequencies is a count of responses for 
        each answer choice in order.
        """
        unit_record = df[self.variable]
        freqs = dict(unit_record.value_counts())
        cts = []
        values = self.scale.get_values(remove_exclusions)
        for k in values:
            if k in freqs:
                cts.append(freqs[k])
            else:
                cts.append(0)
        return((cts, sum(cts), len(unit_record)-sum(cts)))


    def frequency_table(self, df, 
                        show_question=True, 
                        ct=True, 
                        pct=True, 
                        pct_format=".0%", 
                        remove_exclusions=True,
                        show_totals=True, 
                        remove_zero_totals=True, 
                        show_mean=True, 
                        mean_format=".1f", 
                        show_values=True):
        cts, resp, nonresp = self.tally(df, remove_exclusions)
        if resp == 0 and remove_zero_totals:
            return(pd.DataFrame())
        else:
            data = []
            cols = []
            tots = []
            mean = []
            if show_question:
                data.append(self.scale.choices_to_str(remove_exclusions, show_values))
                cols.append("Answer")
                tots.append("Total")
                mean.append("Mean")
            if ct:
                data.append(cts)
                cols.append("Count")
                tots.append(resp)
                mean.append(format(self.mean(df, remove_exclusions), 
                            mean_format))
            if pct:
                l = []
                for x in cts:
                    if resp > 0:
                        l.append(format(x/resp, pct_format))
                    else:
                        l.append("-")
                data.append(l)
                cols.append("%")
                tots.append(format(1, pct_format))
                if not ct:
                    mean.append(format(self.mean(df, remove_exclusions), 
                                mean_format))
                else:
                    mean.append("")
            tbl = pd.DataFrame(data).T
            tbl.columns = cols
            if show_totals:
                tbl.loc[len(tbl)] = tots
            if show_mean:
                tbl.loc[len(tbl)] = mean
            return(tbl)

    def cut_by_json(self, response_set, 
                    sort_by_mean=False, 
                    mark_sig_diffs=False):
        freq_table_options={
            "show_question":True, 
            "ct":True, 
            "pct":False,
            "pct_format":"", 
            "remove_exclusions":True, 
            "show_totals":False,
            "remove_zero_totals":True,
            "show_mean":True,
            "mean_format":"10",
            "show_values":False                         
        }
        var = response_set.grouping_var
        grouped = response_set.get_data()
        group_ct = len(grouped)
        t = self.cut_by(grouped, var, freq_table_options = freq_table_options)
        meancol = "Mean"
        if meancol not in t.columns:
            meancol = "Mean*"
            if mark_sig_diffs:
                t.index=[i + "*" for i in t.index]
        if sort_by_mean:
            t[meancol] = t[meancol].astype(float)
            t.sort([meancol], ascending=[0], inplace=True)
        t.drop(meancol, axis=1, inplace=True)
        totals = t.sum(1).tolist()
        totals = [int(t) for t in totals]
        if self.graph_type(group_ct) == 'clustered_horizontal_bar':
            t = t.T
        t.index = t.index.astype(str)
        t.columns = t.columns.astype(str)
        j = t.to_json(orient="split")
        p = json.loads(j)
        p["totals"] = totals
        return(json.dumps(p))

    def cut_by_question(self, other_question, response_set,
                        freq_table_options={
                            "show_question":True, 
                            "ct":True, 
                            "pct":False,
                            "pct_format":"", 
                            "remove_exclusions":True, 
                            "show_totals":False,
                            "remove_zero_totals":True,
                            "show_mean":False,
                            "mean_format":"",
                            "show_values":False                         
                        },              
                        cut_by_options={
                            "question_label":None,
                            "col_multi_index":False, 
                            "row_multi_index":False 
                        },
                        cut_var_label=None):
        if type(other_question) != SelectOneQuestion:
            raise(Exception("Can only call cut_by_question on a SelectOneQuestion type"))
        df = response_set.data.copy()
        # Here we remove the exclusions for the cut variable, the 
        # exclusions for this question are removed in cut_by, if 
        # appropriate
        if freq_table_options["remove_exclusions"]:
            values_to_drop = other_question.scale.excluded_choices()
            for v in values_to_drop:
                df[other_question.variable].replace(v, np.nan,
                                                    inplace=True)

        groups = df.groupby(other_question.label)
        group_mapping = dict(zip(other_question.scale.values, other_question.scale.choices))

        oth_text = cut_var_label
        if not oth_text:
            oth_text = other_question.text
        return(self.cut_by(groups, group_mapping, oth_text, freq_table_options,
                           **cut_by_options))

    def cut_by(self, groups, cut_var_label,
               group_label_mapping = None,
               freq_table_options = {
                "ct":True, 
                "pct":False,
                "pct_format":"", 
                "remove_exclusions":True, 
                "show_totals":False,
                "remove_zero_totals":True,
                "show_mean":False,
                "mean_format":"",
                "show_values":False
               },
               question_label=None,
               col_multi_index=False, 
               row_multi_index=False):
        freqs = []
        for k, gp in groups:
            t = (self.frequency_table(gp, **freq_table_options))
            if len(t) > 0:
                t.set_index("Answer", inplace=True)
                series = t.ix[:,0]
                if group_label_mapping:
                    series.name = group_label_mapping[k]
                else:
                    series.name = k
                freqs.append(series)
        df = pd.DataFrame(freqs)

        if freq_table_options["show_mean"]:
            if self.compare_groups(groups):
                df.columns = df.columns.tolist()[:-1] + \
                             [df.columns.tolist()[-1]+"*"]

        my_label = question_label
        if not my_label:
            my_label = self.text

        # Add hierarchical index to rows
        if row_multi_index:
            top_index = [cut_var_label]*len(freqs)
            df.index = pd.MultiIndex.from_arrays([top_index, 
                       df.index.tolist()])

        # Add hierarchical index to columns
        if col_multi_index:
            col_top_index = [my_label]*len(self.scale.get_choices(remove_exclusions))

            if show_mean:
                col_top_index += [my_label]
            df.columns = pd.MultiIndex.from_arrays([col_top_index, 
                         df.columns.tolist()])

        return(df)

    def compare_groups(self, groupby, 
                       pval = .05):
        data = [d[self.variable].dropna() for groupname, d in groupby]
        if len(groupby) == 2:
            ts, ps = ttest_ind(*data, equal_var=False)
            return(ps < pval)
        elif len(groupby.groups.keys()) >= 2:
            # ANOVA
            f, p = f_oneway(*data)
            return(p < .05)
        else:
            return(False)

    def freq_table_to_json(self, df):
        t = self.frequency_table(df, 
                                 show_question=True,
                                 ct=True,
                                 pct=True,
                                 pct_format=".9f",
                                 remove_exclusions=True,
                                 show_totals=False,
                                 remove_zero_totals=True,
                                 show_mean=False,
                                 mean_format=".1f",
                                 show_values=False)
        if len(t) == 0:
            return(None)
        t.columns = ["category", "count", "pct"]
        t.set_index('category', inplace=True)
        return(t.to_json(orient="split"))

    def graph_type(self, num_groups=1):
        if num_groups <= 1:
            return('horizontal_bar')
        elif isinstance(self.scale, LikertScale):
            return('diverging_bar')
        else:
            return('clustered_horizontal_bar')

class SelectMultipleQuestion(SelectQuestion):

    def __init__(self, text, vars, choices, label, exclude_from_analysis,
                 matrix=None):
        self.text = text
        self.label = label
        self.variables = vars
        self.matrix = matrix
        self.scale = QuestionScale.create_scale('nominal', choices, 
                                                exclude_from_analysis)

    def get_variable_names(self):
        return(self.variables)

    def reverse_choices(self):
        self.scale.reverse_choices()
        self.variables.reverse()

    def pretty_print(self, 
                     show_choices=True):
        print("{} ({})".format(self.text, self.label))
        if show_choices:
            self.pretty_print_choices()

    def pretty_print_choices(self):
        l = []
        for c, v, x in zip(self.scale.choices, self.variables, 
                           self.scale.exclude_from_analysis):
            if x:
                l.append("{} (X)".format(c))
            else:
                l.append("{} ({})".format(c, v))
        print(", ".join(l))

    def tally(self, df, 
              remove_exclusions=True):
        """
        Returns (list, int1, int2) tuple where list is a count of
        responses for each answer choice. Int1 is the number of 
        respondents, and int2 is the number of nonrespondents.
        """
        vars = self.variables
        if remove_exclusions:
            vars = list(compress(vars, 
                   [not x for x in self.scale.exclude_from_analysis]))
        unit_record = df[vars]
        nonrespondents = 0
        respondents = 0

        cts = [0]*len(vars)
        for index, row in unit_record.iterrows():
            if row.dropna().empty:
                nonrespondents += 1
            else:
                respondents += 1
                ct = 0
                for i, v in row.iteritems():
                    if not np.isnan(v):
                        cts[ct] += 1
                    ct += 1
        return(cts, respondents, nonrespondents)

    def frequency_table(self, df, 
                        show_question=True, 
                        ct=True, 
                        pct_respondents=True, 
                        pct_responses=False, 
                        pct_format=".0%", 
                        remove_exclusions=True,
                        show_totals=True):
        cts, resp, nonresp = self.tally(df, remove_exclusions)
        data = []
        cols = []
        tots = []
        if show_question:
            data.append(self.scale.get_choices(remove_exclusions))
            cols.append("Answer")
            tots.append("Total respondents")
        if ct:
            data.append(cts)
            cols.append("Count")
            tots.append(resp)
        if pct_respondents:
            if resp > 0:
                data.append([format(x/resp, pct_format) for x in cts])
            else:
                data.append(["-" for x in cts])
            cols.append("% of respondents")
            tots.append("")
        if pct_responses:
            if sum(cts) > 0:
                data.append([format(x/sum(cts), pct_format) for x in cts])
            else:
                data.append(["-" for x in cts])
            cols.append("% of responses")
            tots.append("")
        tbl = pd.DataFrame(data).T
        tbl.columns = cols
        if show_totals:
            tbl.loc[len(tbl)] = tots
        return(tbl)

    def cut_by_json(self, response_set, 
                    sort_by_mean=False, 
                    mark_sig_diffs=False):
        freq_table_options={
            "show_question":True, 
            "ct":True, 
            "pct_respondents":False,
            "pct_responses":False, 
            "pct_format":"", 
            "remove_exclusions":True, 
            "show_totals":True                         
        }
        var = response_set.grouping_var
        grouped = response_set.get_data()
        t = self.cut_by(grouped, var, freq_table_options = freq_table_options)
        totals = t[t.columns[-1]].tolist()
        totals = [int(i) for i in totals]
        r = t.iloc[:,:-1].T.to_json(orient="split")
        j = json.loads(r)
        j["totals"] = totals
        return(json.dumps(j))

    def cut_by_question(self, other_question, response_set, 
                        freq_table_options={
                            "show_question":True, 
                            "ct":True, 
                            "pct_respondents":False,
                            "pct_responses":False, 
                            "pct_format":".0%", 
                            "remove_exclusions":True, 
                            "show_totals":False                          
                        },
                        cut_by_options={
                            "question_label":None,
                            "col_multi_index":False, 
                            "row_multi_index":False                            
                        }, 
                        cut_var_label=None):
        if type(other_question) != SelectOneQuestion:
            raise(Exception("Can only call cut_by_question on a SelectOneQuestion type"))
        df = response_set.data.copy()
        # Here we remove the exclusions for the cut variable, the 
        # exclusions for this question are removed in cut_by, if 
        # appropriate
        if freq_table_options["remove_exclusions"]:
            values_to_drop = [v for v, e in zip(other_question.scale.values, 
                              other_question.scale.exclude_from_analysis) if e]
            for v in values_to_drop:
                df[other_question.variable].replace(v, np.nan,
                                                      inplace=True)

        groups = df.groupby(other_question.label)
        group_mapping = dict(zip(other_question.scale.values, other_question.scale.choices))

        oth_text = cut_var_label
        if not oth_text:
            oth_text = other_question.text
        return(self.cut_by(groups, group_mapping, oth_text, freq_table_options,
               **cut_by_options))

    def cut_by(self, groups, cut_var_label,
               group_label_mapping = None, 
               freq_table_options={
                    "show_question":True, 
                    "ct":True, 
                    "pct_respondents":False,
                    "pct_responses":False, 
                    "pct_format":".0%", 
                    "remove_exclusions":True, 
                    "show_totals":False
               }, 
               question_label=None, 
               col_multi_index=False, 
               row_multi_index=False):
        freqs = []
        for k, gp in groups:
            t = (self.frequency_table(gp, **freq_table_options))
            t.set_index("Answer", inplace=True)
            series = t.ix[:,0]
            if group_label_mapping:
                series.name = group_label_mapping[k]
            else:
                series.name = k
            freqs.append(series)
        df = pd.DataFrame(freqs)

        my_label = question_label
        if not my_label:
            my_label = self.text

        # Add significance flags
        sigs = self.compare_groups(groups, freq_table_options["remove_exclusions"])
        newcols = []
        for s, i in zip(sigs, df.columns.tolist()):
            if s:
                newcols.append(i + "*")
            else:
                newcols.append(i)

        # Add hierarchical index to rows
        if row_multi_index:
            top_index = [cut_var_label]*len(groups)
            df.index = pd.MultiIndex.from_arrays([top_index, 
                       df.index.tolist()])

        # Add hierarchical index to columns
        if col_multi_index:
            col_top_index = [my_label]*len(self.scale.choices)
            df.columns = pd.MultiIndex.from_arrays([col_top_index, 
                         newcols])
        return(df)

    def compare_groups(self, groupby, 
                       remove_exclusions=True, 
                       pval = .05):
        groupnames = groupby.groups.keys()
        obs_by_cut = []
        ct_by_cut = []
        for k, df in groupby:
            freqs, tot_resp, tot_nonresp = self.tally(df, remove_exclusions)
            obs_by_cut.append(freqs)
            ct_by_cut.append(tot_resp)
        choice_totals = [sum(x) for x in zip(*obs_by_cut)]
        exp_prop_per_choice = [t/sum(ct_by_cut) for t in choice_totals]
        sigs = []
        for f_obs, choice_tot, p_choice in zip(zip(*obs_by_cut), 
            choice_totals, exp_prop_per_choice):
            f_exp = [p_choice * ct for ct in ct_by_cut]
            chisq, p = chisquare(f_obs, f_exp)
            sigs.append(p < pval)
        return(sigs)

    def freq_table_to_json(self, df):
        t = self.frequency_table(df, True, True, True, False, ".9f", True, False)
        t.columns = ["category", "count", "pct"]
        t.set_index('category', inplace=True)
        return(t.to_json(orient="split"))

    def graph_type(self, num_groups=1):
        if num_groups <= 1:
            return('horizontal_bar')
        else:
            return('clustered_horizontal_bar')



"""
Codebook
--------
Represents information about all the questions in a survey and
their possible responses.
"""

from collections import OrderedDict

class Codebook:

    def __init__(self, survey_title, questions_list):
        """
        store all of the questions in an OrderedDict
        """
        self.survey_title = survey_title
        self.questions = OrderedDict()
        for q in questions_list:
            if q.label in self.questions:
                k = 1
                while "{}{}".format(q.label, k) in self.questions:
                    k += 1
                self.questions["{}{}".format(q.label, k)] = q
            else:
                self.questions[q.label] = q

    def get_question(self, qname):
        if qname in self.questions:
            return(self.questions[qname])
        else:
            raise KeyError("Question not found: {}".format(qname))

    def get_questions(self):
        return([v for k, v in self.questions.items()])

    def list_questions(self):
        print("\n".join([t for t, q in self.questions.items()]))

    def pretty_print(self):
        print("Survey: {}".format(self.survey_title))
        for k, v in self.questions.items():
            v.pretty_print()
            print()
        return

    def get_variable_names(self):
        """
        Returns a list of all of the variable names associated with the
        questions in the codebook.
        """
        vars = []
        for q in self.get_questions():
            vars += q.get_variable_names()
        return(vars)



from itertools import compress

class QuestionScale():

    @staticmethod
    def create_scale(type, choices, exclude_from_analysis, 
                     values=None, 
                     midpoint=None):
        if type=='nominal':
            return(NominalScale(choices, exclude_from_analysis))
        elif type =='ordinal':
            return(OrdinalScale(choices, exclude_from_analysis, values))
        elif type =='likert':
            return(LikertScale(choices, exclude_from_analysis, values, midpoint))
        else:
            raise(Exception("Invalid scale type: {}".format(type)))

    def __init__(self, choices, exclude_from_analysis):
        self.choices = choices
        self.exclude_from_analysis = exclude_from_analysis

    def __eq__(self, other): 
        return (self.choices == other.choices and 
                self.exclude_from_analysis == other.exclude_from_analysis)

    @staticmethod
    def change_scale(oldscale, new_type, 
                     new_values=None,
                     new_midpoint=None):
        if hasattr(oldscale, 'values') and new_values==None:
            new_values = oldscale.values
        if hasattr(oldscale, 'midpoint') and new_midpoint==None:
            new_midpoint = oldscale.midpoint
        return(QuestionScale.create_scale(new_type, oldscale.choices, 
            oldscale.exclude_from_analysis, new_values, new_midpoint))


    def reverse_choices(self):
        self.choices.reverse()
        self.exclude_from_analysis.reverse()

    def get_choices(self, 
                    remove_exclusions=True):
        choices = self.choices
        if remove_exclusions:
            choices = list(compress(choices, 
                      [not x for x in self.exclude_from_analysis]))
        return(choices)

    def exclude_choices_from_analysis(self, choices):
        new_excl = []
        for c, e in zip(self.choices, self.exclude_from_analysis):
            if c in choices:
                new_excl.append(True)
            else:
                new_excl.append(e)
        self.exclude_from_analysis = new_excl

    def excluded_choices(self):
        x = list(compress(self.choices, 
                      [x for x in self.exclude_from_analysis]))
        return(x)

class NominalScale(QuestionScale):

    def __init__(self, choices, exclude_from_analysis):
        super().__init__(choices, exclude_from_analysis)


class OrdinalScale(QuestionScale):

    def __init__(self, choices, exclude_from_analysis, values):
        super().__init__(choices, exclude_from_analysis)
        self.values = values

    def __eq__(self, other):
        if super().__eq__(other):
            return(self.values == other.values)
        else:
            return(False)

    def reverse_choices(self):
        super().reverse_choices()
        self.values.reverse()

    def get_values(self, 
                   remove_exclusions=True):
        values = self.values
        if remove_exclusions:
            values = list(compress(values, 
                      [not x for x in self.exclude_from_analysis]))
        return(values)

    def choices_to_str(self, 
                       remove_exclusions=False, 
                       show_values=True):
        choices = self.get_choices(remove_exclusions)
        values = self.get_values(remove_exclusions)
        if show_values:
            new_choices = []
            for c, v, x in zip(choices, values, self.exclude_from_analysis):
                if x:
                    new_choices.append("{} (X)".format(c))
                else:
                    new_choices.append("{} ({})".format(c, v))
            choices = new_choices
        return(choices)


class LikertScale(OrdinalScale):

    def __init__(self, choices, exclude_from_analysis, values, 
                 midpoint=None):
        super().__init__(choices, exclude_from_analysis, values)
        ct = len(self.exclude_from_analysis) - sum(self.exclude_from_analysis)
        if not midpoint:
            self.midpoint = ct/2
        else:
            if midpoint > ct or midpoint < 0:
                raise(
                    Exception("Invalid midpoint {} for {} point scale".format(midpoint, ct
                    ))
                )
            else:
                self.midpoint = midpoint

    def __eq__(self, other):
        if super().__eq__(other):
            return(self.midpoint == other.midpoint)
        else:
            return(False)

    def exclude_choices_from_analysis(self, choices):
        super().exclude_choices_from_analysis(choices)
        ct = len(self.exclude_from_analysis) - sum(self.exclude_from_analysis)
        self.midpoint = ct/2




"""
Parses a Qualtrics .qsf file and creates a codebook from it.
"""

import json
import re
import logging
from pprint import pprint
from bs4 import BeautifulSoup
from surveyhelper import SelectOneQuestion, SelectMultipleQuestion, \
SelectOneMatrixQuestion, SelectMultipleMatrixQuestion, Codebook

class QsfParser:

    def __init__(self, qsf_filename, 
                 exclude_trash = True):
        with open(qsf_filename) as qsf_file:
            qsf = json.load(qsf_file)
        self.qsf = qsf
        self.exclude_trash = exclude_trash

    @staticmethod
    def remove_html(text):
        soup = BeautifulSoup(text, "html.parser")
        [s.extract() for s in soup('style')]
        return(soup.get_text().strip())

    def create_codebook(self, 
                        title = None):
        q = self.create_questions()
        if title == None:
            title = self.get_survey_title()
        return(Codebook(title, q))

    def get_survey_title(self):
        return(self.qsf['SurveyEntry']['SurveyName'])

    def pretty_print(self):
        pprint(self.qsf)
        return

    def get_block_order(self):
        """
        Return a list of block ids in the order they appear in the
        survey
        """
        flow_info = [e['Payload']['Flow'] 
                     for e in self.qsf['SurveyElements']
                     if e['Element'] == 'FL']
        if len(flow_info) != 1:
            raise(Exception("Invalid flow specification"))
        block_order = [e['ID'] for e in flow_info[0]
                       if e['Type'] in ['Standard', 'Block']]
        return(block_order)

    def get_block_dict(self):
        """
        Returns a dictionary mapping block id to the dictionary
        containing all the block info
        """
        block_info = [e['Payload'] for e in self.qsf['SurveyElements']
                      if e['Element'] == 'BL']
        if len(block_info) != 1:
            raise(Exception("Invalid block specification"))
        blocks = block_info[0]
        blocks_by_id = {}

        # It appears that sometimes blocks is a list containing one
        # or more dicts, and sometimes it's already a dict at this point
        if type(blocks) == list:
            for block_dict in blocks:
                blocks_by_id[block_dict['ID']] = block_dict

        else:
            for k, v in blocks.items():
                blocks_by_id[v['ID']] = v

        return(blocks_by_id)

    def get_question_order(self):
        """
        Returns a list of question ids, in the same order they 
        appear in the survey
        """
        block_order = self.get_block_order()
        blocks_by_id = self.get_block_dict()

        ordered_qids = []
        for k in block_order:
            data = blocks_by_id[k]
            if ((data['Type'] == 'Trash' and self.exclude_trash) or 
            'BlockElements' not in data):
                continue
            qids = [e['QuestionID'] for e in data['BlockElements']
                    if e['Type'] == 'Question']
            ordered_qids += qids
        return(ordered_qids)

    def get_dynamic_choice_json(self, locator):
        other_q = self.qsf['SurveyElements']
        m = re.search('q://(.+?)/', locator)
        if m:
            qid = m.group(1)
        else:
            logging.info("No dynamic choices found")
            return({})
        for j in self.qsf['SurveyElements']:
            if j['Element'] == 'SQ' and j['Payload']['QuestionID'] == qid:
                return(j)
        return({})


    def get_question_json(self, qids):
        """
        Given a list of Qualtrics question ids, extract the 
        corresponding parsed JSON. Returns a dict, mapping QID to to
        parsed JSON dict
        """
        q = [e for e in self.qsf['SurveyElements'] 
             if e['Element'] == 'SQ' 
             and e['Payload']['QuestionID'] in qids]
        questions = {}
        for i in q:
            # if a question pipes in dynamic choices from another question,
            # pull in those choices
            p = i['Payload']
            if 'DynamicChoices' in p and 'Locator' in p['DynamicChoices']:
                locator = p['DynamicChoices']['Locator']
                xtra_choices = self.get_dynamic_choice_json(locator)
                i['Payload']['DynamicChoiceJson'] = xtra_choices['Payload']
            questions[i['Payload']['QuestionID']] = i
        return(questions)

    def create_questions(self):
        """
        From what I can tell about the qsf format, the question types we
        want to extract are the ones where "QuestionType" == "MC",
        "Matrix", "TE" (text entry), or "SBS" (side by side)
        """
        
        qids = self.get_question_order()
        qid_to_json = self.get_question_json(qids)

        questions = []
        for id in qids:
            json = qid_to_json[id]['Payload']
            if (json['QuestionType'] == 'MC' and 
            json['Selector'] in ['SAVR', 'SAHR', 'DL', 'SACOL']):
                questions.append(self.build_sqsr(json))
            elif (json['QuestionType'] == 'MC' and 
            json['Selector'] in ['MACOL', 'MAVR']):
                questions.append(self.build_sqmr(json))
            elif (json['QuestionType'] == 'Matrix' and
            json['Selector'] == 'Likert' and json['SubSelector'] ==
            'SingleAnswer'):
                questions.append(self.build_mqsr(json))
            elif json['QuestionType'] == 'Slider':
                questions.append(self.build_slider_mqsr(json))
            elif (json['QuestionType'] == 'Matrix' and
            json['Selector'] == 'Likert' and json['SubSelector'] ==
            'MultipleAnswer'):
                questions.append(self.build_mqmr(json))
            else:
                txt = json['QuestionText']
                qtype = json['QuestionType']
                select = json['Selector']
                logging.info("Skipping question {}\n\nType: {}\nSelector: {}")
        return([q for q in questions if q is not None])

    def _recode_exclusions(self, exclusions, recode):
        exclusions_rcd = {}
        for k, v in exclusions.items():
            if v != "No":
                continue
            elif k in recode:
                exclusions_rcd[int(recode[k])] = True
            else:
                exclusions_rcd[int(k)] = True
        return(exclusions_rcd)

    def _parse_recode_values(self, json):
        if 'RecodeValues' in json:
            recode = json['RecodeValues']
        else:
            recode = {}
        return(recode)

    def _parse_analyze_choices(self, json):
        if 'AnalyzeChoices' in json:
            exclude = json['AnalyzeChoices']
        else:
            exclude = {}
        return(exclude)   


    def build_sr_question(self, text, label, choices, recode, var,
        provides_int, exclusions):
        """
        """
        txt_cln = QsfParser.remove_html(text)
        if choices == []:
            return(None)
        choices_cln = dict([(k, QsfParser.remove_html(v['Display'])) 
                           for k, v in choices.items()])
        choices_rcd = {}
        for k, v in recode.items():
            choices_rcd[int(v)] = choices_cln[k]
        for k in choices_cln.keys() - recode.keys():
            choices_rcd[int(k)] = choices_cln[k]
        exclusions_rcd = self._recode_exclusions(exclusions, recode)

        choices_text = []
        values = []
        for k in sorted(choices_rcd.keys()):
            values.append(k)
            choices_text.append(choices_rcd[k])
        exclude = []
        for v in values:
            if v in exclusions_rcd:
                exclude.append(True)
            else:
                exclude.append(False)
        return(SelectOneQuestion(txt_cln, var, choices_text, label, 
               values, exclude))

    def build_sqsr(self, json):
        recode = self._parse_recode_values(json)
        exclude = self._parse_analyze_choices(json)
        q = self.build_sr_question(json['QuestionText'], 
            json['DataExportTag'], json['Choices'], recode, 
            json['DataExportTag'], True, exclude)
        return(q)

    def build_sr_question_from_id(self, id, json, tag, recode, exclude):
        text = json['Choices'][str(id)]['Display']
        if 'ChoiceDataExportTags' in json and json['ChoiceDataExportTags']:
            var = json['ChoiceDataExportTags'][str(id)]
        else:
            var = "{}_{}".format(tag, id)
        q = self.build_sr_question(text, tag, json['Answers'], 
            recode, var, True, exclude)
        return(q)

    def build_mqsr(self, json):
        stem_text = QsfParser.remove_html(json['QuestionText'])
        tag = json['DataExportTag']
        recode = self._parse_recode_values(json)
        exclude = self._parse_analyze_choices(json)

        questions = []
        for k in json['ChoiceOrder']:
            q = self.build_sr_question_from_id(k, json, tag, recode, exclude)
            questions.append(q)
        if 'DynamicChoiceJson' in json:
            other_json = json['DynamicChoiceJson']
            for choice_id in other_json['ChoiceOrder']:
                text = other_json['Choices'][str(choice_id)]['Display']
                var = "{}_x{}".format(tag, choice_id)
                q = self.build_sr_question(text, tag, json['Answers'], 
                    recode, var, True, exclude)
                questions.append(q)
        return(SelectOneMatrixQuestion(stem_text, tag, questions))

    def build_slider_mqsr(self, json):
        stem_text = QsfParser.remove_html(json['QuestionText'])
        tag = json['DataExportTag']
        recode = self._parse_recode_values(json)
        exclude = self._parse_analyze_choices(json)

        questions = []
        dec = int(json["Configuration"]["NumDecimals"])
        if dec == 0:
            step = 1
        else:
            step = 1/(dec*10)
        slide_min = json["Configuration"]["CSSliderMin"]
        slide_max = json["Configuration"]["CSSliderMax"] + step
        nums = list(range(slide_min, slide_max, step))
        num_map = {}
        for num in nums:
            num_map[num] = {"Display" : str(num)}
        json['Answers'] = num_map

        for k in json['ChoiceOrder']:
            q = self.build_sr_question_from_id(k, json, tag, recode, exclude)
            questions.append(q)
        if 'DynamicChoiceJson' in json:
            raise(Exception("Slider questions with dynamic choices aren't handled yet"))
        mat_q = SelectOneMatrixQuestion(stem_text, tag, questions)
        mat_q.change_scale('ordinal')
        return(mat_q)

    def build_mr_question(self, text, label, choices, recode, var_base,
        provides_int, exclusions):
        """
        """
        text = QsfParser.remove_html(text)
        choices_cln = dict([(k, QsfParser.remove_html(v['Display'])) 
                           for k, v in choices.items()])
        choices_rcd = {}
        recode_inverse = {}
        for k, v in recode.items():
            choices_rcd[int(v)] = choices_cln[k]
            recode_inverse[int(v)] = int(k)
        for k in choices_cln.keys() - recode.keys():
            choices_rcd[int(k)] = choices_cln[k]
            recode_inverse[int(k)] = int(k)

        exclusions_rcd = self._recode_exclusions(exclusions, recode)
        ans_choices = []
        variables = []
        exclude = []
        for c in sorted(choices_rcd.keys()):
            ans_choices.append(choices_rcd[c])
            orig_pos = recode_inverse[c]
            variables.append("{}_{}".format(var_base, orig_pos))
            if c in exclusions_rcd:
                exclude.append(True)
            else:
                exclude.append(False)
        return(SelectMultipleQuestion(text, variables, ans_choices,
            label, exclude))


    def build_sqmr(self, json):
        recode = self._parse_recode_values(json)
        exclude = self._parse_analyze_choices(json)
        q = self.build_mr_question(json['QuestionText'], 
            json['DataExportTag'], json['Choices'], recode, 
            json['DataExportTag'], True, exclude)
        return(q)

    def build_mqmr(self, json):
        stem_text = QsfParser.remove_html(json['QuestionText'])
        tag = json['DataExportTag']
        recode = self._parse_recode_values(json)
        exclude = self._parse_analyze_choices(json)

        children = []
        for k in json['ChoiceOrder']:
            text = json['Choices'][str(k)]['Display']
            if 'ChoiceDataExportTags' in json and json['ChoiceDataExportTags']:
                var = json['ChoiceDataExportTags'][str(k)]
            else:
                var = "{}_{}".format(tag, k)
            q = self.build_mr_question(text, var, json['Answers'], 
                recode, var, True, exclude)
            children.append(q)
        return(SelectMultipleMatrixQuestion(stem_text, tag, children))

                


from setuptools import setup

def readme():
	with open('README.rst') as f:
		return(f.read())

setup(name='surveyhelper',
	  version='0.1.1.dev1',
	  description="Survey package for reporting on survey data from Qualtrics.",
	  url="http://github.com/cwade/surveyhelper",
	  author="cwade",
	  author_email="pysurveyhelper@gmail.com",
	  license='MIT',
	  packages=['surveyhelper'],
	  install_requires=['beautifulsoup4', 'pandas', 'jinja2', 'unidecode', 'scipy', 'pyyaml'],
	  zip_safe=False)


"""
FrequencyReport
--------
Provides utilities for producing a survey frequency report.
"""

from jinja2 import Environment, FileSystemLoader
import yaml, math
from itertools import compress
from unidecode import unidecode
from surveyhelper.question import SelectQuestion, SelectOneMatrixQuestion

class FrequencyReport:

    def __init__(self, response_set, config_file):
        with open(config_file, 'r') as ymlfile:
            cfg = yaml.load(ymlfile)

        self.template_dir = cfg['output']['template_dir']
        self.freq_template = cfg['output']['template_file']
        self.report_title = cfg['report_data']['title']
        self.response_set = response_set

    def create_report(self,
                      report_file,
                      sort_by_mean=False,
                      mark_sig_diffs=False):
        env = Environment(loader=FileSystemLoader(self.template_dir),
                  extensions=['jinja2.ext.with_'])
        template = env.get_template(self.freq_template)
        outfile = open(report_file, 'w+')
        
        matched_questions = self.response_set.matched_questions
        data_groups = self.response_set.get_data()

        if len(data_groups) > 1:
            questions = self.grouped_questions_data(matched_questions, 
                                                    data_groups, 
                                                    sort_by_mean,
                                                    mark_sig_diffs)
        else:
            questions = self.ungrouped_questions_data(matched_questions, 
                                                      self.response_set.data,
                                                      sort_by_mean)
        t = template.render(count=len(self.response_set.data),
                            survey_title=self.report_title,
                            questions=questions)
        outfile.write(unidecode(t))


    def ungrouped_questions_data(self, matched_questions, data, sort_by_mean):
        questions = []
        for q in matched_questions:
            scale = q.get_scale()
            if scale and hasattr(scale, 'midpoint'):
                midpoint = scale.midpoint
            else:
                midpoint = None

            # If this is a matrix question with more than 7 choices, split it
            # into multiple questions
            if (isinstance(q, SelectOneMatrixQuestion) and len(q.get_scale().choices) > 7):
                barheight = math.floor(min(800 / len(q.get_scale().choices), 30))
                for s in q.questions:
                    table = s.freq_table_to_json(data)
                    group_names = ['']
                    text = "{} &mdash; {}".format(q.text, s.text)
                    questions.append((
                        text,
                        [table],
                        [table],
                        group_names,
                        s.graph_type(1),
                        midpoint,
                        barheight
                    ))                  
            else:
                j = q.freq_table_to_json(data)
                if j != '':
                    questions.append((
                                    q.text,
                                    [j],
                                    [j],
                                    [''],
                                    q.graph_type(1),
                                    midpoint,
                                    40
                    ))
        return(questions)

    def grouped_questions_data(self, matched_questions, data_groups, 
                               sort_by_mean, mark_sig_diffs):
        questions = []
        for q in matched_questions:
            scale = q.get_scale()
            if scale and hasattr(scale, 'midpoint'):
                midpoint = scale.midpoint
            else:
                midpoint = None

            # If we have a lot of comparison groups or questions in the 
            # matrix, break the graph into multiple graphs
            if isinstance(q, SelectOneMatrixQuestion) and ((len(data_groups) * 
                len(q.questions)) > 36):
                for sub_q in q.questions:
                    table = sub_q.cut_by_json(self.response_set, 
                                              sort_by_mean=sort_by_mean,
                                              mark_sig_diffs=mark_sig_diffs)
                    text = "{} &mdash; {}".format(q.text, sub_q.text)
                    questions.append((
                        text,
                        [table],
                        [table],
                        [''],
                        q.graph_type(len(data_groups)),
                        midpoint,
                        22
                    ))
            # If this is a matrix question with more than 7 answer choices, split it
            # into multiple questions
            elif (isinstance(q, SelectOneMatrixQuestion) and len(q.get_scale().choices) > 7):
                barheight = math.floor(min(800 / len(q.get_scale().choices), 30))
                for s in q.questions:
                    table = s.cut_by_json(self.response_set, 
                                          sort_by_mean=sort_by_mean,
                                          mark_sig_diffs=mark_sig_diffs)
                    group_names = [n for n, df in data_groups]
                    text = "{} &mdash; {}".format(q.text, s.text)
                    questions.append((
                        text,
                        [table],
                        [table],
                        group_names,
                        s.graph_type(len(data_groups)),
                        midpoint,
                        barheight
                    ))
            elif isinstance(q, SelectQuestion) and len(q.get_scale().choices) > 7:
                group_names = []
                json = []
                tables = []
                for name, df in data_groups:
                    j = q.freq_table_to_json(df)
                    if j != '':
                        group_names.append(name)
                        json.append(j)
                        tables.append(j)
                questions.append((
                                q.text,
                                tables,
                                json,
                                group_names,
                                q.graph_type(1),
                                midpoint,
                                22
                ))
            else:
                freq_tables = []
                freq_table_json = []
                group_names = []
                # Matrix question (potentially with groups) that doesn't need
                # to be split apart
                if isinstance(q, SelectOneMatrixQuestion):
                    j = q.cut_by_json(self.response_set, 
                                      sort_by_mean=sort_by_mean,
                                      mark_sig_diffs=mark_sig_diffs)
                    if j != '':
                        freq_tables.append(j)
                        freq_table_json.append(j)
                        group_names.append('')
                    barheight = math.floor(min(800 / ((len(data_groups) * len(q.questions))), 40))
                # Select question with some cut variable
                elif isinstance(q, SelectQuestion):
                    table = q.cut_by_json(self.response_set, 
                                          sort_by_mean=sort_by_mean,
                                          mark_sig_diffs=mark_sig_diffs)
                    freq_tables.append(table)
                    freq_table_json.append(table)
                    group_names.append('')
                    barheight = math.floor(min(800 / len(data_groups), 40))

                questions.append((
                                q.text,
                                freq_tables,
                                freq_table_json,
                                group_names,
                                q.graph_type(len(data_groups)),
                                midpoint,
                                barheight
                                ))
        return(questions)


